@@ -71,9 +71,9 @@ struct ArgumentProcessingState
   bool found_md_or_mmd_opt = false;
   bool found_Wa_a_opt = false;
 
-  std::string explicit_language;       // As specified with -x.
-  std::string input_charset_option;    // -finput-charset=...
-  std::string last_seen_msvc_z_option; // /Z7, /Zi or /ZI
+  std::string explicit_language;             // As specified with -x.
+  std::string input_charset_option;          // -finput-charset=...
+  std::string last_seen_msvc_z_debug_option; // /Z7, /Zi or /ZI
 
   // Is the dependency file set via -Wp,-M[M]D,target or -MFtarget?
   OutputDepOrigin output_dep_origin = OutputDepOrigin::none;
@@ -277,6 +277,14 @@ make_dash_option(const Config& config, const std::string& arg)
   return new_arg;
 }
 
+bool
+is_msvc_z_debug_option(std::string_view arg)
+{
+  static const char* debug_options[] = {"-Z7", "-ZI", "-Zi"};
+  return std::find(std::begin(debug_options), std::end(debug_options), arg)
+         != std::end(debug_options);
+}
+
 // Returns std::nullopt if the option wasn't recognized, otherwise the error
 // code (with Statistic::none for "no error").
 std::optional<Statistic>
@@ -608,8 +616,9 @@ process_option_arg(const Context& ctx,
     return Statistic::none;
   }
 
-  if (config.is_compiler_group_msvc() && util::starts_with(arg, "-Z")) {
-    state.last_seen_msvc_z_option = args[i];
+  if (config.is_compiler_group_msvc() && !config.is_compiler_group_clang()
+      && is_msvc_z_debug_option(arg)) {
+    state.last_seen_msvc_z_debug_option = args[i];
     state.common_args.push_back(args[i]);
     return Statistic::none;
   }
@@ -1167,10 +1176,11 @@ process_args(Context& ctx)
     return Statistic::unsupported_compiler_option;
   }
 
-  if (!state.last_seen_msvc_z_option.empty()
-      && state.last_seen_msvc_z_option.substr(2) != "7") {
+  if (!state.last_seen_msvc_z_debug_option.empty()
+      && state.last_seen_msvc_z_debug_option.substr(2) != "7") {
     // /Zi and /ZI are unsupported, but /Z7 is fine.
-    LOG("Compiler option {} is unsupported", state.last_seen_msvc_z_option);
+    LOG("Compiler option {} is unsupported",
+        state.last_seen_msvc_z_debug_option);
     return Statistic::unsupported_compiler_option;
   }
 
