@@ -38,7 +38,7 @@ get_dev_null_path()
 }
 
 bool
-is_absolute_path(nonstd::string_view path)
+is_absolute_path(std::string_view path)
 {
 #ifdef _WIN32
   if (path.length() >= 2 && path[1] == ':'
@@ -50,8 +50,11 @@ is_absolute_path(nonstd::string_view path)
 }
 
 bool
-path_starts_with(nonstd::string_view path, nonstd::string_view prefix)
+path_starts_with(std::string_view path, std::string_view prefix)
 {
+  if (path.empty()) {
+    return false;
+  }
   for (size_t i = 0, j = 0; i < path.length() && j < prefix.length();
        ++i, ++j) {
 #ifdef _WIN32
@@ -70,22 +73,26 @@ path_starts_with(nonstd::string_view path, nonstd::string_view prefix)
     if (path[i] == '\\' && prefix[j] == '/') {
       continue;
     }
-#endif
+    if (std::tolower(path[i]) != std::tolower(prefix[j])) {
+      return false;
+    }
+#else
     if (path[i] != prefix[j]) {
       return false;
     }
+#endif
   }
   return true;
 }
 
 std::vector<std::string>
-split_path_list(nonstd::string_view path_list)
+split_path_list(std::string_view path_list)
 {
   return Util::split_into_strings(path_list, k_path_delimiter);
 }
 
 std::string
-to_absolute_path(nonstd::string_view path)
+to_absolute_path(std::string_view path)
 {
   if (util::is_absolute_path(path)) {
     return std::string(path);
@@ -96,7 +103,7 @@ to_absolute_path(nonstd::string_view path)
 }
 
 std::string
-to_absolute_path_no_drive(nonstd::string_view path)
+to_absolute_path_no_drive(std::string_view path)
 {
   std::string abs_path = to_absolute_path(path);
 #ifdef _WIN32
